@@ -1,6 +1,4 @@
-//! Creates a HTTP request to the official Rust website.
-
-#![feature(async_await)]
+//! Sends an HTTP request to the Rust website.
 
 use async_std::task;
 
