@@ -1,10 +1,9 @@
-use std::fs;
 use std::path::Path;
 
 use crate::io;
 use crate::task::blocking;
 
-/// Removes an directory and all of its contents.
+/// Removes a directory and all of its contents.
 ///
 /// This function is an async version of [`std::fs::remove_dir_all`].
 ///
@@ -12,10 +11,11 @@ use crate::task::blocking;
 ///
 /// # Errors
 ///
-/// An error will be returned in the following situations (not an exhaustive list):
+/// An error will be returned in the following situations:
 ///
-/// * `path` is not a directory.
-/// * The current process lacks permissions to remove directory at `path`.
+/// * `path` is not an existing and empty directory.
+/// * The current process lacks permissions to remove the directory.
+/// * Some other I/O error occurred.
 ///
 /// # Examples
 ///
@@ -24,11 +24,11 @@ use crate::task::blocking;
 /// #
 /// use async_std::fs;
 ///
-/// fs::remove_dir_all("./some/dir").await?;
+/// fs::remove_dir_all("./some/directory").await?;
 /// #
 /// # Ok(()) }) }
 /// ```
 pub async fn remove_dir_all<P: AsRef<Path>>(path: P) -> io::Result<()> {
     let path = path.as_ref().to_owned();
-    blocking::spawn(async move { fs::remove_dir_all(path) }).await
+    blocking::spawn(async move { std::fs::remove_dir_all(path) }).await
 }
