@@ -1,10 +1,9 @@
-use std::fs;
 use std::path::Path;
 
 use crate::io;
 use crate::task::blocking;
 
-/// Removes a file from the filesystem.
+/// Removes a file.
 ///
 /// This function is an async version of [`std::fs::remove_file`].
 ///
@@ -12,10 +11,11 @@ use crate::task::blocking;
 ///
 /// # Errors
 ///
-/// An error will be returned in the following situations (not an exhaustive list):
+/// An error will be returned in the following situations:
 ///
-/// * `path` is not a file.
-/// * The current process lacks permissions to remove file at `path`.
+/// * `path` does not point to an existing file.
+/// * The current process lacks permissions to remove the file.
+/// * Some other I/O error occurred.
 ///
 /// # Examples
 ///
@@ -30,5 +30,5 @@ use crate::task::blocking;
 /// ```
 pub async fn remove_file<P: AsRef<Path>>(path: P) -> io::Result<()> {
     let path = path.as_ref().to_owned();
-    blocking::spawn(async move { fs::remove_file(path) }).await
+    blocking::spawn(async move { std::fs::remove_file(path) }).await
 }
