@@ -1,21 +1,26 @@
-use std::fs::{self, Metadata};
 use std::path::Path;
 
+use crate::fs::Metadata;
 use crate::io;
 use crate::task::blocking;
 
-/// Queries the metadata for a path without following symlinks.
+/// Reads metadata for a path without following symbolic links.
+///
+/// If you want to follow symbolic links before reading metadata of the target file or directory,
+/// use [`metadata`] instead.
 ///
 /// This function is an async version of [`std::fs::symlink_metadata`].
 ///
+/// [`metadata`]: fn.metadata.html
 /// [`std::fs::symlink_metadata`]: https://doc.rust-lang.org/std/fs/fn.symlink_metadata.html
 ///
 /// # Errors
 ///
-/// An error will be returned in the following situations (not an exhaustive list):
+/// An error will be returned in the following situations:
 ///
-/// * `path` does not exist.
-/// * The current process lacks permissions to query metadata for `path`.
+/// * `path` does not point to an existing file or directory.
+/// * The current process lacks permissions to read metadata for the path.
+/// * Some other I/O error occurred.
 ///
 /// # Examples
 ///
@@ -30,5 +35,5 @@ use crate::task::blocking;
 /// ```
 pub async fn symlink_metadata<P: AsRef<Path>>(path: P) -> io::Result<Metadata> {
     let path = path.as_ref().to_owned();
-    blocking::spawn(async move { fs::symlink_metadata(path) }).await
+    blocking::spawn(async move { std::fs::symlink_metadata(path) }).await
 }
