@@ -3,10 +3,9 @@ use std::pin::Pin;
 use std::sync::Mutex;
 
 use cfg_if::cfg_if;
-use futures::future;
-use futures::io::{AsyncRead, Initializer};
+use futures_io::{AsyncRead, Initializer};
 
-use crate::future::Future;
+use crate::future::{self, Future};
 use crate::task::{blocking, Context, Poll};
 
 /// Constructs a new handle to the standard input of the current process.
@@ -130,7 +129,7 @@ impl Stdin {
                         }
                     }
                     // Poll the asynchronous operation the stdin is currently blocked on.
-                    State::Busy(task) => *state = futures::ready!(Pin::new(task).poll(cx)),
+                    State::Busy(task) => *state = futures_core::ready!(Pin::new(task).poll(cx)),
                 }
             }
         })
@@ -182,7 +181,7 @@ impl AsyncRead for Stdin {
                     }
                 }
                 // Poll the asynchronous operation the stdin is currently blocked on.
-                State::Busy(task) => *state = futures::ready!(Pin::new(task).poll(cx)),
+                State::Busy(task) => *state = futures_core::ready!(Pin::new(task).poll(cx)),
             }
         }
     }
