@@ -1,11 +1,12 @@
 use std::io;
-use std::net::{self, Ipv4Addr, Ipv6Addr, SocketAddr, ToSocketAddrs};
+use std::net::SocketAddr;
 
 use cfg_if::cfg_if;
-use futures::future;
+use std::net::{Ipv4Addr, Ipv6Addr};
 
-use crate::net::driver::IoHandle;
-use crate::task::Poll;
+use crate::future;
+use crate::net::driver::Watcher;
+use crate::net::ToSocketAddrs;
 
 /// A UDP socket.
 ///
@@ -45,12 +46,7 @@ use crate::task::Poll;
 /// ```
 #[derive(Debug)]
 pub struct UdpSocket {
-    io_handle: IoHandle<mio::net::UdpSocket>,
-
-    #[cfg(unix)]
-    raw_fd: std::os::unix::io::RawFd,
-    // #[cfg(windows)]
-    // raw_socket: std::os::windows::io::RawSocket,
+    watcher: Watcher<mio::net::UdpSocket>,
 }
 
 impl UdpSocket {
@@ -75,22 +71,12 @@ impl UdpSocket {
     pub async fn bind<A: ToSocketAddrs>(addr: A) -> io::Result<UdpSocket> {
         let mut last_err = None;
 
-        for addr in addr.to_socket_addrs()? {
+        for addr in addr.to_socket_addrs().await? {
             match mio::net::UdpSocket::bind(&addr) {
                 Ok(mio_socket) => {
-                    #[cfg(unix)]
-                    let socket = UdpSocket {
-                        raw_fd: mio_socket.as_raw_fd(),
-                        io_handle: IoHandle::new(mio_socket),
-                    };
-
-                    #[cfg(windows)]
-                    let socket = UdpSocket {
-                        // raw_socket: mio_socket.as_raw_socket(),
-                        io_handle: IoHandle::new(mio_socket),
-                    };
-
-                    return Ok(socket);
+                    return Ok(UdpSocket {
+                        watcher: Watcher::new(mio_socket),
+                    });
                 }
                 Err(err) => last_err = Some(err),
             }
@@ -122,7 +108,7 @@ impl UdpSocket {
     /// # Ok(()) }) }
     /// ```
     pub fn local_addr(&self) -> io::Result<SocketAddr> {
-        self.io_handle.get_ref().local_addr()
+        self.watcher.get_ref().local_addr()
     }
 
     /// Sends data on the socket to the given address.
@@ -152,7 +138,7 @@ impl UdpSocket {
     /// # Ok(()) }) }
     /// ```
     pub async fn send_to<A: ToSocketAddrs>(&self, buf: &[u8], addrs: A) -> io::Result<usize> {
-        let addr = match addrs.to_socket_addrs()?.next() {
+        let addr = match addrs.to_socket_addrs().await?.next() {
             Some(addr) => addr,
             None => {
                 return Err(io::Error::new(
@@ -163,16 +149,8 @@ impl UdpSocket {
         };
 
         future::poll_fn(|cx| {
-            futures::ready!(self.io_handle.poll_writable(cx)?);
-
-            match self.io_handle.get_ref().send_to(buf, &addr) {
-                Ok(n) => Poll::Ready(Ok(n)),
-                Err(ref err) if err.kind() == io::ErrorKind::WouldBlock => {
-                    self.io_handle.clear_writable(cx)?;
-                    Poll::Pending
-                }
-                Err(err) => Poll::Ready(Err(err)),
-            }
+            self.watcher
+                .poll_write_with(cx, |inner| inner.send_to(buf, &addr))
         })
         .await
     }
@@ -198,16 +176,8 @@ impl UdpSocket {
     /// ```
     pub async fn recv_from(&self, buf: &mut [u8]) -> io::Result<(usize, SocketAddr)> {
         future::poll_fn(|cx| {
-            futures::ready!(self.io_handle.poll_readable(cx)?);
-
-            match self.io_handle.get_ref().recv_from(buf) {
-                Ok(n) => Poll::Ready(Ok(n)),
-                Err(ref err) if err.kind() == io::ErrorKind::WouldBlock => {
-                    self.io_handle.clear_readable(cx)?;
-                    Poll::Pending
-                }
-                Err(err) => Poll::Ready(Err(err)),
-            }
+            self.watcher
+                .poll_read_with(cx, |inner| inner.recv_from(buf))
         })
         .await
     }
@@ -237,8 +207,9 @@ impl UdpSocket {
     pub async fn connect<A: ToSocketAddrs>(&self, addrs: A) -> io::Result<()> {
         let mut last_err = None;
 
-        for addr in addrs.to_socket_addrs()? {
-            match self.io_handle.get_ref().connect(addr) {
+        for addr in addrs.to_socket_addrs().await? {
+            // TODO(stjepang): connect on the blocking pool
+            match self.watcher.get_ref().connect(addr) {
                 Ok(()) => return Ok(()),
                 Err(err) => last_err = Some(err),
             }
@@ -279,19 +250,7 @@ impl UdpSocket {
     /// # Ok(()) }) }
     /// ```
     pub async fn send(&self, buf: &[u8]) -> io::Result<usize> {
-        future::poll_fn(|cx| {
-            futures::ready!(self.io_handle.poll_writable(cx)?);
-
-            match self.io_handle.get_ref().send(buf) {
-                Ok(n) => Poll::Ready(Ok(n)),
-                Err(ref err) if err.kind() == io::ErrorKind::WouldBlock => {
-                    self.io_handle.clear_writable(cx)?;
-                    Poll::Pending
-                }
-                Err(err) => Poll::Ready(Err(err)),
-            }
-        })
-        .await
+        future::poll_fn(|cx| self.watcher.poll_write_with(cx, |inner| inner.send(buf))).await
     }
 
     /// Receives data from the socket.
@@ -314,19 +273,7 @@ impl UdpSocket {
     /// # Ok(()) }) }
     /// ```
     pub async fn recv(&self, buf: &mut [u8]) -> io::Result<usize> {
-        future::poll_fn(|cx| {
-            futures::ready!(self.io_handle.poll_readable(cx)?);
-
-            match self.io_handle.get_ref().recv(buf) {
-                Ok(n) => Poll::Ready(Ok(n)),
-                Err(ref err) if err.kind() == io::ErrorKind::WouldBlock => {
-                    self.io_handle.clear_readable(cx)?;
-                    Poll::Pending
-                }
-                Err(err) => Poll::Ready(Err(err)),
-            }
-        })
-        .await
+        future::poll_fn(|cx| self.watcher.poll_read_with(cx, |inner| inner.recv(buf))).await
     }
 
     /// Gets the value of the `SO_BROADCAST` option for this socket.
@@ -335,14 +282,14 @@ impl UdpSocket {
     ///
     /// [`set_broadcast`]: #method.set_broadcast
     pub fn broadcast(&self) -> io::Result<bool> {
-        self.io_handle.get_ref().broadcast()
+        self.watcher.get_ref().broadcast()
     }
 
     /// Sets the value of the `SO_BROADCAST` option for this socket.
     ///
     /// When enabled, this socket is allowed to send packets to a broadcast address.
     pub fn set_broadcast(&self, on: bool) -> io::Result<()> {
-        self.io_handle.get_ref().set_broadcast(on)
+        self.watcher.get_ref().set_broadcast(on)
     }
 
     /// Gets the value of the `IP_MULTICAST_LOOP` option for this socket.
@@ -351,7 +298,7 @@ impl UdpSocket {
     ///
     /// [`set_multicast_loop_v4`]: #method.set_multicast_loop_v4
     pub fn multicast_loop_v4(&self) -> io::Result<bool> {
-        self.io_handle.get_ref().multicast_loop_v4()
+        self.watcher.get_ref().multicast_loop_v4()
     }
 
     /// Sets the value of the `IP_MULTICAST_LOOP` option for this socket.
@@ -362,7 +309,7 @@ impl UdpSocket {
     ///
     /// This may not have any affect on IPv6 sockets.
     pub fn set_multicast_loop_v4(&self, on: bool) -> io::Result<()> {
-        self.io_handle.get_ref().set_multicast_loop_v4(on)
+        self.watcher.get_ref().set_multicast_loop_v4(on)
     }
 
     /// Gets the value of the `IP_MULTICAST_TTL` option for this socket.
@@ -371,7 +318,7 @@ impl UdpSocket {
     ///
     /// [`set_multicast_ttl_v4`]: #method.set_multicast_ttl_v4
     pub fn multicast_ttl_v4(&self) -> io::Result<u32> {
-        self.io_handle.get_ref().multicast_ttl_v4()
+        self.watcher.get_ref().multicast_ttl_v4()
     }
 
     /// Sets the value of the `IP_MULTICAST_TTL` option for this socket.
@@ -384,7 +331,7 @@ impl UdpSocket {
     ///
     /// This may not have any affect on IPv6 sockets.
     pub fn set_multicast_ttl_v4(&self, ttl: u32) -> io::Result<()> {
-        self.io_handle.get_ref().set_multicast_ttl_v4(ttl)
+        self.watcher.get_ref().set_multicast_ttl_v4(ttl)
     }
 
     /// Gets the value of the `IPV6_MULTICAST_LOOP` option for this socket.
@@ -393,7 +340,7 @@ impl UdpSocket {
     ///
     /// [`set_multicast_loop_v6`]: #method.set_multicast_loop_v6
     pub fn multicast_loop_v6(&self) -> io::Result<bool> {
-        self.io_handle.get_ref().multicast_loop_v6()
+        self.watcher.get_ref().multicast_loop_v6()
     }
 
     /// Sets the value of the `IPV6_MULTICAST_LOOP` option for this socket.
@@ -404,7 +351,7 @@ impl UdpSocket {
     ///
     /// This may not have any affect on IPv4 sockets.
     pub fn set_multicast_loop_v6(&self, on: bool) -> io::Result<()> {
-        self.io_handle.get_ref().set_multicast_loop_v6(on)
+        self.watcher.get_ref().set_multicast_loop_v6(on)
     }
 
     /// Gets the value of the `IP_TTL` option for this socket.
@@ -413,7 +360,7 @@ impl UdpSocket {
     ///
     /// [`set_ttl`]: #method.set_ttl
     pub fn ttl(&self) -> io::Result<u32> {
-        self.io_handle.get_ref().ttl()
+        self.watcher.get_ref().ttl()
     }
 
     /// Sets the value for the `IP_TTL` option on this socket.
@@ -421,7 +368,7 @@ impl UdpSocket {
     /// This value sets the time-to-live field that is used in every packet sent
     /// from this socket.
     pub fn set_ttl(&self, ttl: u32) -> io::Result<()> {
-        self.io_handle.get_ref().set_ttl(ttl)
+        self.watcher.get_ref().set_ttl(ttl)
     }
 
     /// Executes an operation of the `IP_ADD_MEMBERSHIP` type.
@@ -449,7 +396,7 @@ impl UdpSocket {
     /// # Ok(()) }) }
     /// ```
     pub fn join_multicast_v4(&self, multiaddr: &Ipv4Addr, interface: &Ipv4Addr) -> io::Result<()> {
-        self.io_handle
+        self.watcher
             .get_ref()
             .join_multicast_v4(multiaddr, interface)
     }
@@ -478,7 +425,7 @@ impl UdpSocket {
     /// # Ok(()) }) }
     /// ```
     pub fn join_multicast_v6(&self, multiaddr: &Ipv6Addr, interface: u32) -> io::Result<()> {
-        self.io_handle
+        self.watcher
             .get_ref()
             .join_multicast_v6(multiaddr, interface)
     }
@@ -489,7 +436,7 @@ impl UdpSocket {
     ///
     /// [`join_multicast_v4`]: #method.join_multicast_v4
     pub fn leave_multicast_v4(&self, multiaddr: &Ipv4Addr, interface: &Ipv4Addr) -> io::Result<()> {
-        self.io_handle
+        self.watcher
             .get_ref()
             .leave_multicast_v4(multiaddr, interface)
     }
@@ -500,30 +447,19 @@ impl UdpSocket {
     ///
     /// [`join_multicast_v6`]: #method.join_multicast_v6
     pub fn leave_multicast_v6(&self, multiaddr: &Ipv6Addr, interface: u32) -> io::Result<()> {
-        self.io_handle
+        self.watcher
             .get_ref()
             .leave_multicast_v6(multiaddr, interface)
     }
 }
 
-impl From<net::UdpSocket> for UdpSocket {
+impl From<std::net::UdpSocket> for UdpSocket {
     /// Converts a `std::net::UdpSocket` into its asynchronous equivalent.
-    fn from(socket: net::UdpSocket) -> UdpSocket {
+    fn from(socket: std::net::UdpSocket) -> UdpSocket {
         let mio_socket = mio::net::UdpSocket::from_socket(socket).unwrap();
-
-        #[cfg(unix)]
-        let socket = UdpSocket {
-            raw_fd: mio_socket.as_raw_fd(),
-            io_handle: IoHandle::new(mio_socket),
-        };
-
-        #[cfg(windows)]
-        let socket = UdpSocket {
-            // raw_socket: mio_socket.as_raw_socket(),
-            io_handle: IoHandle::new(mio_socket),
-        };
-
-        socket
+        UdpSocket {
+            watcher: Watcher::new(mio_socket),
+        }
     }
 }
 
@@ -543,19 +479,19 @@ cfg_if! {
     if #[cfg(any(unix, feature = "docs"))] {
         impl AsRawFd for UdpSocket {
             fn as_raw_fd(&self) -> RawFd {
-                self.raw_fd
+                self.watcher.get_ref().as_raw_fd()
             }
         }
 
         impl FromRawFd for UdpSocket {
             unsafe fn from_raw_fd(fd: RawFd) -> UdpSocket {
-                net::UdpSocket::from_raw_fd(fd).into()
+                std::net::UdpSocket::from_raw_fd(fd).into()
             }
         }
 
         impl IntoRawFd for UdpSocket {
             fn into_raw_fd(self) -> RawFd {
-                self.raw_fd
+                self.watcher.into_inner().into_raw_fd()
             }
         }
     }
