@@ -33,7 +33,7 @@ pub struct Once<T> {
     value: Option<T>,
 }
 
-impl<T: Unpin> futures::Stream for Once<T> {
+impl<T: Unpin> futures_core::stream::Stream for Once<T> {
     type Item = T;
 
     fn poll_next(mut self: Pin<&mut Self>, _: &mut Context<'_>) -> Poll<Option<T>> {
