@@ -24,14 +24,21 @@
 #[doc(inline)]
 pub use std::task::{Context, Poll, Waker};
 
-pub use local::{AccessError, LocalKey};
-pub use pool::{block_on, current, spawn, Builder};
+pub use block_on::block_on;
+pub use builder::Builder;
+pub use pool::spawn;
 pub use sleep::sleep;
 pub use task::{JoinHandle, Task, TaskId};
+pub use task_local::{AccessError, LocalKey};
+pub use worker::current;
 
-mod local;
+mod block_on;
+mod builder;
 mod pool;
 mod sleep;
+mod sleepers;
 mod task;
+mod task_local;
+mod worker;
 
 pub(crate) mod blocking;
